@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import os
 import re
+import warnings
 from functools import lru_cache
 from pathlib import Path
 from tempfile import TemporaryDirectory
@@ -14,7 +15,7 @@ from unearth import Link, vcs_support
 from pdm import termui
 from pdm.builders import EditableBuilder, WheelBuilder
 from pdm.compat import importlib_metadata as im
-from pdm.exceptions import BuildError, CandidateNotFound
+from pdm.exceptions import BuildError, CandidateNotFound, InvalidPyVersion
 from pdm.models.requirements import (
     FileRequirement,
     Requirement,
@@ -23,6 +24,7 @@ from pdm.models.requirements import (
     filter_requirements_with_extras,
 )
 from pdm.models.setup import Setup
+from pdm.models.specifiers import PySpecSet
 from pdm.project.metadata import MutableMetadata, SetupDistribution
 from pdm.utils import (
     cached_property,
@@ -198,9 +200,15 @@ class Candidate:
             return self._requires_python
         if self.link:
             requires_python = self.link.requires_python
-            if requires_python and requires_python.isdigit():
-                requires_python = f">={requires_python},<{int(requires_python) + 1}"
-            self._requires_python = requires_python
+            if requires_python is not None:
+                if requires_python.isdigit():
+                    requires_python = f">={requires_python},<{int(requires_python) + 1}"
+                try:  # ensure the specifier is valid
+                    PySpecSet(requires_python)
+                except InvalidPyVersion:
+                    pass
+                else:
+                    self._requires_python = requires_python
         return self._requires_python or ""
 
     @requires_python.setter
@@ -483,7 +491,15 @@ class PreparedCandidate:
             ).prepare_metadata(metadir_parent)
         except BuildError:
             termui.logger.warn("Failed to build package, try parsing project files.")
-            setup = Setup.from_directory(self._unpacked_dir)
+            try:
+                setup = Setup.from_directory(self._unpacked_dir)
+            except Exception:
+                message = (
+                    "Failed to parse the project files, dependencies may be missing"
+                )
+                termui.logger.warn(message)
+                warnings.warn(message, RuntimeWarning)
+                setup = Setup()
             return SetupDistribution(setup)
         else:
             return im.PathDistribution(Path(self._metadata_dir))
