@@ -5,22 +5,22 @@ import sys
 from functools import lru_cache, wraps
 from typing import TYPE_CHECKING, Any, Callable, Iterable, Mapping, TypeVar, cast
 
-from packaging.version import parse as parse_version
-from html5lib import parse
+from unearth import Link
 
 from pdm import termui
-from pdm._types import CandidateInfo, Package, SearchResult, Source
-from pdm.exceptions import CandidateInfoNotFound, CandidateNotFound, CorruptedCacheError
-from pdm.models.candidates import Candidate
+from pdm.exceptions import CandidateInfoNotFound, CandidateNotFound
+from pdm.models.candidates import Candidate, make_candidate
 from pdm.models.requirements import (
     Requirement,
     filter_requirements_with_extras,
     parse_requirement,
 )
-from pdm.models.specifiers import PySpecSet, get_specifier
-from pdm.utils import allow_all_wheels, normalize_name, url_without_fragments
+from pdm.models.search import SearchResultParser
+from pdm.models.specifiers import PySpecSet
+from pdm.utils import normalize_name, url_without_fragments
 
 if TYPE_CHECKING:
+    from pdm._types import CandidateInfo, SearchResult, Source
     from pdm.models.environment import Environment
 
 ALLOW_ALL_PYTHON = PySpecSet()
@@ -80,7 +80,11 @@ class BaseRepository:
             # XXX: If the requirement has extras, add the original candidate
             # (without extras) as its dependency. This ensures the same package with
             # different extras resolve to the same version.
-            self_req = dataclasses.replace(candidate.req, extras=None, marker=None)
+            self_req = dataclasses.replace(
+                candidate.req.as_pinned_version(candidate.version),
+                extras=None,
+                marker=None,
+            )
             reqs.append(self_req)
         # Store the metadata on the candidate for caching
         candidate.requires_python = requires_python
@@ -90,6 +94,31 @@ class BaseRepository:
     def _find_candidates(self, requirement: Requirement) -> Iterable[Candidate]:
         raise NotImplementedError
 
+    def is_this_package(self, requirement: Requirement) -> bool:
+        """Whether the requirement is the same as this package"""
+        project = self.environment.project
+        return (
+            requirement.is_named
+            and project.name is not None
+            and requirement.key == normalize_name(project.name)
+        )
+
+    def make_this_candidate(self, requirement: Requirement) -> Candidate | None:
+        """Make a candidate for this package, or None if the requirement doesn't match.
+        In this case the finder will look for a candidate from the package sources
+        """
+        project = self.environment.project
+        assert project.name
+        link = Link.from_path(project.root)  # type: ignore
+        version: str = project.meta.version
+        if (
+            not version
+            or requirement.specifier
+            and not requirement.specifier.contains(version, True)
+        ):
+            return None
+        return make_candidate(requirement, project.name, version, link)
+
     def find_candidates(
         self,
         requirement: Requirement,
@@ -101,12 +130,13 @@ class BaseRepository:
         """
         # `allow_prereleases` is None means leave it to specifier to decide whether to
         # include prereleases
+
+        if self.is_this_package(requirement):
+            candidate = self.make_this_candidate(requirement)
+            if candidate is not None:
+                return [candidate]
         requires_python = requirement.requires_python & self.environment.python_requires
-        cans = sorted(
-            self._find_candidates(requirement),
-            key=lambda c: (parse_version(c.version), c.link.is_wheel),  # type: ignore
-            reverse=True,
-        )
+        cans = list(self._find_candidates(requirement))
         applicable_cans = [
             c
             for c in cans
@@ -177,11 +207,8 @@ class BaseRepository:
     def _get_dependencies_from_cache(self, candidate: Candidate) -> CandidateInfo:
         try:
             result = self._candidate_info_cache.get(candidate)
-        except CorruptedCacheError:
-            self._candidate_info_cache.clear()
-            raise CandidateInfoNotFound(candidate)
         except KeyError:
-            raise CandidateInfoNotFound(candidate)
+            raise CandidateInfoNotFound(candidate) from None
         return result
 
     @cache_result
@@ -192,7 +219,7 @@ class BaseRepository:
         summary = prepared.metadata.metadata["Summary"]
         return deps, requires_python, summary
 
-    def get_hashes(self, candidate: Candidate) -> dict[str, str] | None:
+    def get_hashes(self, candidate: Candidate) -> dict[Link, str] | None:
         """Get hashes of all possible installable candidates
         of a given package version.
         """
@@ -204,22 +231,27 @@ class BaseRepository:
             return None
         if candidate.hashes:
             return candidate.hashes
-        req = dataclasses.replace(
-            candidate.req, specifier=get_specifier(f"=={candidate.version}")
-        )
+        req = candidate.req.as_pinned_version(candidate.version)
         if candidate.req.is_file_or_url:
             matching_candidates: Iterable[Candidate] = [candidate]
         else:
-            matching_candidates = self.find_candidates(req, True)
+            matching_candidates = self.find_candidates(req, ignore_requires_python=True)
         result: dict[str, str] = {}
         with self.environment.get_finder(self.sources) as finder:
-            self._hash_cache.session = finder.session  # type: ignore
             for c in matching_candidates:
-                link = c.prepare(self.environment).ireq.link
-                if not link or link.is_vcs:
+                assert c.link is not None
+                # Prepare the candidate to replace vars in the link URL
+                prepared_link = c.prepare(self.environment).link
+                if (
+                    not prepared_link
+                    or prepared_link.is_vcs
+                    or prepared_link.is_file
+                    and prepared_link.file_path.is_dir()
+                ):
                     continue
-                result[link.filename] = self._hash_cache.get_hash(link)
-
+                result[c.link] = self._hash_cache.get_hash(
+                    prepared_link, finder.session
+                )
         return result or None
 
     def dependency_generators(self) -> Iterable[Callable[[Candidate], CandidateInfo]]:
@@ -257,7 +289,7 @@ class PyPIRepository(BaseRepository):
             if proc_url.endswith("/simple")
         ]
         with self.environment.get_finder(sources) as finder:
-            session = finder.session  # type: ignore
+            session = finder.session
             for prefix in url_prefixes:
                 json_url = f"{prefix}/pypi/{candidate.name}/{candidate.version}/json"
                 resp = session.get(json_url)
@@ -289,10 +321,10 @@ class PyPIRepository(BaseRepository):
     @lru_cache()
     def _find_candidates(self, requirement: Requirement) -> Iterable[Candidate]:
         sources = self.get_filtered_sources(requirement)
-        with self.environment.get_finder(sources, True) as finder, allow_all_wheels():
+        with self.environment.get_finder(sources, True) as finder:
             cans = [
                 Candidate.from_installation_candidate(c, requirement)
-                for c in finder.find_all_candidates(requirement.project_name)
+                for c in finder.find_all_packages(requirement.project_name)
             ]
         if not cans:
             raise CandidateNotFound(
@@ -303,7 +335,6 @@ class PyPIRepository(BaseRepository):
 
     def search(self, query: str) -> SearchResult:
         pypi_simple = self.sources[0]["url"].rstrip("/")
-        results = []
 
         if pypi_simple.endswith("/simple"):
             search_url = pypi_simple[:-6] + "search"
@@ -315,34 +346,19 @@ class PyPIRepository(BaseRepository):
             resp = session.get(search_url, params={"q": query})
             if resp.status_code == 404:
                 self.environment.project.core.ui.echo(
-                    termui.yellow(
-                        f"{pypi_simple!r} doesn't support '/search' endpoint, fallback "
-                        f"to {self.DEFAULT_INDEX_URL!r} now.\n"
-                        "This may take longer depending on your network condition."
-                    ),
+                    f"{pypi_simple!r} doesn't support '/search' endpoint, fallback "
+                    f"to {self.DEFAULT_INDEX_URL!r} now.\n"
+                    "This may take longer depending on your network condition.",
                     err=True,
+                    style="yellow",
                 )
                 resp = session.get(
                     f"{self.DEFAULT_INDEX_URL}/search", params={"q": query}
                 )
+            parser = SearchResultParser()
             resp.raise_for_status()
-            content = parse(resp.content, namespaceHTMLElements=False)
-
-        for result in content.findall(".//*[@class='package-snippet']"):
-            name = result.find("h3/*[@class='package-snippet__name']").text
-            version = result.find("h3/*[@class='package-snippet__version']").text
-
-            if not name or not version:
-                continue
-
-            description = result.find("p[@class='package-snippet__description']").text
-            if not description:
-                description = ""
-
-            result = Package(name, version, description)
-            results.append(result)
-
-        return results
+            parser.feed(resp.text)
+            return parser.results
 
 
 class LockedRepository(BaseRepository):
@@ -354,7 +370,7 @@ class LockedRepository(BaseRepository):
     ) -> None:
         super().__init__(sources, environment)
         self.packages: dict[tuple, Candidate] = {}
-        self.file_hashes: dict[tuple[str, str], dict[str, str]] = {}
+        self.file_hashes: dict[tuple[str, str], dict[Link, str]] = {}
         self.candidate_info: dict[tuple, CandidateInfo] = {}
         self._read_lockfile(lockfile)
 
@@ -374,7 +390,7 @@ class LockedRepository(BaseRepository):
                 if k not in ("dependencies", "requires_python", "summary")
             }
             req = Requirement.from_req_dict(package_name, req_dict)
-            can = Candidate(req, name=package_name, version=version)
+            can = make_candidate(req, name=package_name, version=version)
             can_id = self._identify_candidate(can)
             self.packages[can_id] = can
             candidate_info: CandidateInfo = (
@@ -386,7 +402,7 @@ class LockedRepository(BaseRepository):
 
         for key, hashes in lockfile.get("metadata", {}).get("files", {}).items():
             self.file_hashes[tuple(key.split(None, 1))] = {  # type: ignore
-                item["file"]: item["hash"] for item in hashes
+                Link(item["url"]): item["hash"] for item in hashes if "url" in item
             }
 
     def _identify_candidate(self, candidate: Candidate) -> tuple:
@@ -422,6 +438,11 @@ class LockedRepository(BaseRepository):
         allow_prereleases: bool | None = None,
         ignore_requires_python: bool = False,
     ) -> Iterable[Candidate]:
+        if self.is_this_package(requirement):
+            candidate = self.make_this_candidate(requirement)
+            if candidate is not None:
+                yield candidate
+                return
         for key, info in self.candidate_info.items():
             if key[0] != requirement.identify():
                 continue
@@ -435,7 +456,7 @@ class LockedRepository(BaseRepository):
             can.req = requirement
             yield can
 
-    def get_hashes(self, candidate: Candidate) -> dict[str, str] | None:
+    def get_hashes(self, candidate: Candidate) -> dict[Link, str] | None:
         assert candidate.name
         return self.file_hashes.get(
             (normalize_name(candidate.name), candidate.version or "")
