@@ -6,41 +6,35 @@ import json
 import os
 import re
 import secrets
-import sys
 import urllib.parse as urlparse
 import warnings
 from pathlib import Path
-from typing import Any, Sequence, Type, TypeVar, cast
+from typing import TYPE_CHECKING, Any, Sequence, TypeVar, cast
 
 from packaging.markers import InvalidMarker
 from packaging.requirements import InvalidRequirement
+from packaging.requirements import Requirement as PackageRequirement
 from packaging.specifiers import SpecifierSet
-from pkg_resources import Requirement as PackageRequirement
-from pkg_resources import RequirementParseError, safe_name
+from packaging.utils import parse_sdist_filename, parse_wheel_filename
+from packaging.version import parse as parse_version
+from unearth import Link
 
-from pdm._types import RequirementDict
+from pdm.compat import Distribution
 from pdm.exceptions import ExtrasWarning, RequirementError
 from pdm.models.markers import Marker, get_marker, split_marker_extras
-from pdm.models.pip_shims import (
-    InstallRequirement,
-    Link,
-    install_req_from_editable,
-    install_req_from_line,
-    path_to_url,
-    url_to_path,
-)
 from pdm.models.setup import Setup
 from pdm.models.specifiers import PySpecSet, get_specifier
 from pdm.utils import (
     add_ssh_scheme_to_git_uri,
-    parse_name_version_from_wheel,
+    normalize_name,
+    path_to_url,
+    url_to_path,
     url_without_fragments,
 )
 
-if sys.version_info >= (3, 8):
-    from importlib.metadata import Distribution
-else:
-    from importlib_metadata import Distribution
+if TYPE_CHECKING:
+    from pdm._types import RequirementDict
+
 
 VCS_SCHEMA = ("git", "hg", "svn", "bzr")
 _vcs_req_re = re.compile(
@@ -62,13 +56,11 @@ def strip_extras(line: str) -> tuple[str, tuple[str, ...] | None]:
     match = re.match(r"^(.+?)(?:\[([^\]]+)\])?$", line)
     assert match is not None
     name, extras_str = match.groups()
-    extras = (
-        tuple(set(e.strip() for e in extras_str.split(","))) if extras_str else None
-    )
+    extras = tuple({e.strip() for e in extras_str.split(",")}) if extras_str else None
     return name, extras
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(eq=False)
 class Requirement:
     """Base class of a package requirement.
     A requirement is a (virtual) specification of a package which contains
@@ -89,33 +81,32 @@ class Requirement:
 
     @property
     def project_name(self) -> str | None:
-        return safe_name(self.name) if self.name else None  # type: ignore
+        return normalize_name(self.name, lowercase=False) if self.name else None
 
     @property
     def key(self) -> str | None:
         return self.project_name.lower() if self.project_name else None
 
     @property
-    def version(self) -> str | None:
+    def is_pinned(self) -> bool:
         if not self.specifier:
-            return None
+            return False
 
-        is_pinned = len(self.specifier) == 1 and next(
-            iter(self.specifier)
-        ).operator in (
+        return len(self.specifier) == 1 and next(iter(self.specifier)).operator in (
             "==",
             "===",
         )
-        if is_pinned:
-            return next(iter(self.specifier)).version
-        return None
 
-    @version.setter
-    def version(self, v: str) -> None:
-        if not v or v == "*":
-            self.specifier = SpecifierSet()
-        else:
-            self.specifier = get_specifier(f"=={v}")
+    def as_pinned_version(self: T, other_version: str | None) -> T:
+        """Return a new requirement with the given pinned version."""
+        if self.is_pinned or not other_version:
+            return self
+        version = parse_version(other_version)
+        normalized = str(version)
+        if version.local:
+            # Remove the local part to accept wider range of prereleases.
+            normalized = normalized.rsplit("+", 1)[0]
+        return dataclasses.replace(self, specifier=get_specifier(f"=={normalized}"))
 
     def _hash_key(self) -> tuple:
         return (
@@ -128,7 +119,7 @@ class Requirement:
         return hash(self._hash_key())
 
     def __eq__(self, o: object) -> bool:
-        return isinstance(o, Requirement) and hash(self) == hash(o)
+        return isinstance(o, Requirement) and self._hash_key() == o._hash_key()
 
     @functools.lru_cache(maxsize=None)
     def identify(self) -> str:
@@ -144,7 +135,7 @@ class Requirement:
         return self.as_line()
 
     @classmethod
-    def create(cls: Type[T], **kwargs: Any) -> T:
+    def create(cls: type[T], **kwargs: Any) -> T:
         if "marker" in kwargs:
             try:
                 kwargs["marker"] = get_marker(kwargs["marker"])
@@ -160,7 +151,7 @@ class Requirement:
         return cls(**kwargs)
 
     @classmethod
-    def from_dist(cls, dist: Distribution) -> "Requirement":
+    def from_dist(cls, dist: Distribution) -> Requirement:
         direct_url_json = dist.read_text("direct_url.json")
         if direct_url_json is not None:
             direct_url = json.loads(direct_url_json)
@@ -184,7 +175,7 @@ class Requirement:
         )
 
     @classmethod
-    def from_req_dict(cls, name: str, req_dict: RequirementDict) -> "Requirement":
+    def from_req_dict(cls, name: str, req_dict: RequirementDict) -> Requirement:
         if isinstance(req_dict, str):  # Version specifier only.
             return NamedRequirement(name=name, specifier=get_specifier(req_dict))
         for vcs in VCS_SCHEMA:
@@ -211,7 +202,7 @@ class Requirement:
     def as_line(self) -> str:
         raise NotImplementedError
 
-    def matches(self, line: str, editable_match: bool = True) -> bool:
+    def matches(self, line: str) -> bool:
         """Return whether the passed in PEP 508 string
         is the same requirement as this one.
         """
@@ -219,25 +210,10 @@ class Requirement:
             req = parse_requirement(line.split("-e ", 1)[-1], True)
         else:
             req = parse_requirement(line, False)
-        return self.key == req.key and (
-            not editable_match or self.editable == req.editable
-        )
-
-    def as_ireq(self, **kwargs: Any) -> InstallRequirement:
-        line_for_req = self.as_line()
-        try:
-            if self.editable:
-                line_for_req = line_for_req[3:].strip()
-                ireq = install_req_from_editable(line_for_req, **kwargs)
-            else:
-                ireq = install_req_from_line(line_for_req, **kwargs)
-        except Exception as e:
-            raise RequirementError(e)
-        ireq.req = self  # type: ignore
-        return ireq
+        return self.key == req.key
 
     @classmethod
-    def from_pkg_requirement(cls, req: PackageRequirement) -> "Requirement":
+    def from_pkg_requirement(cls, req: PackageRequirement) -> Requirement:
         kwargs = {
             "name": req.name,
             "extras": req.extras,
@@ -257,17 +233,14 @@ class Requirement:
         return ""
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(eq=False)
 class NamedRequirement(Requirement):
-    def __hash__(self) -> int:
-        return hash(self._hash_key())
-
     def as_line(self) -> str:
         extras = f"[{','.join(sorted(self.extras))}]" if self.extras else ""
         return f"{self.project_name}{extras}{self.specifier}{self._format_marker()}"
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(eq=False)
 class FileRequirement(Requirement):
     url: str = ""
     path: Path | None = None
@@ -284,11 +257,8 @@ class FileRequirement(Requirement):
     def _hash_key(self) -> tuple:
         return super()._hash_key() + (self.url, self.editable)
 
-    def __hash__(self) -> int:
-        return hash(self._hash_key())
-
     @classmethod
-    def create(cls: Type[T], **kwargs: Any) -> T:
+    def create(cls: type[T], **kwargs: Any) -> T:
         if kwargs.get("path"):
             kwargs["path"] = Path(kwargs["path"].replace("${PROJECT_ROOT}", "."))
         return super().create(**kwargs)  # type: ignore
@@ -335,18 +305,38 @@ class FileRequirement(Requirement):
     def is_local_dir(self) -> bool:
         return self.is_local and cast(Path, self.path).is_dir()
 
+    def as_file_link(self) -> Link:
+        url = self.get_full_url()
+        # only subdirectory is useful in a file link
+        if self.subdirectory:
+            url += f"#subdirectory={self.subdirectory}"
+        return Link(url)
+
+    def get_full_url(self) -> str:
+        return url_without_fragments(self.url)
+
     def as_line(self) -> str:
         project_name = f"{self.project_name}" if self.project_name else ""
-        extras = f"[{','.join(sorted(self.extras))}]" if self.extras else ""
+        extras = (
+            f"[{','.join(sorted(self.extras))}]"
+            if self.extras and self.project_name
+            else ""
+        )
         marker = self._format_marker()
-        url = url_without_fragments(self.url)
-        if self.editable or self.subdirectory:
-            fragments = f"egg={project_name}{extras}"
-            if self.subdirectory:
-                fragments = f"{fragments}&subdirectory={self.subdirectory}"
-            return f"{'-e ' if self.editable else ''}{url}#{fragments}{marker}"
+        if marker:
+            marker = f" {marker}"
+        url = self.get_full_url()
+        fragments = []
+        if self.subdirectory:
+            fragments.append(f"subdirectory={self.subdirectory}")
+        if self.editable:
+            if project_name:
+                fragments.insert(0, f"egg={project_name}{extras}")
+            fragment_str = ("#" + "&".join(fragments)) if fragments else ""
+            return f"-e {url}{fragment_str}{marker}"
         delimiter = " @ " if project_name else ""
-        return f"{project_name}{extras}{delimiter}{url}{marker}"
+        fragment_str = ("#" + "&".join(fragments)) if fragments else ""
+        return f"{project_name}{extras}{delimiter}{url}{fragment_str}{marker}"
 
     def _parse_name_from_url(self) -> None:
         parsed = urlparse.urlparse(self.url)
@@ -361,13 +351,17 @@ class FileRequirement(Requirement):
                 urlparse.unquote(url_without_fragments(self.url))
             )
             if filename.endswith(".whl"):
-                self.name, _ = parse_name_version_from_wheel(filename)
+                self.name, *_ = parse_wheel_filename(filename)
             else:
-                match = _egg_info_re.match(filename)
-                # Filename is like `<name>-<version>.tar.gz`, where name will be
-                # extracted and version will be left to be determined from the metadata.
-                if match:
-                    self.name = match.group(1)
+                try:
+                    self.name, *_ = parse_sdist_filename(filename)
+                except ValueError:
+                    match = _egg_info_re.match(filename)
+                    # Filename is like `<name>-<version>.tar.gz`, where name will be
+                    # extracted and version will be left to be determined from
+                    # the metadata.
+                    if match:
+                        self.name = match.group(1)
 
     def _check_installable(self) -> None:
         assert self.path
@@ -380,36 +374,24 @@ class FileRequirement(Requirement):
         self.name = result.name
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(eq=False)
 class VcsRequirement(FileRequirement):
     vcs: str = ""
     ref: str | None = None
     revision: str | None = None
 
-    def __hash__(self) -> int:
-        return hash(self._hash_key())
-
     def __post_init__(self) -> None:
         super().__post_init__()
         if not self.vcs:
             self.vcs = self.url.split("+", 1)[0]
 
-    def as_line(self) -> str:
-        project_name = f"{self.project_name}" if self.project_name else ""
-        extras = f"[{','.join(sorted(self.extras))}]" if self.extras else ""
-        marker = self._format_marker()
-        url = self.url
+    def get_full_url(self) -> str:
+        url = super().get_full_url()
         if self.revision and not self.editable:
             url += f"@{self.revision}"
         elif self.ref:
             url += f"@{self.ref}"
-        if self.editable or self.subdirectory:
-            fragments = f"egg={project_name}{extras}"
-            if self.subdirectory:
-                fragments = f"{fragments}&subdirectory={self.subdirectory}"
-            return f"{'-e ' if self.editable else ''}{url}#{fragments}{marker}"
-        delimiter = " @ " if project_name else ""
-        return f"{project_name}{extras}{delimiter}{url}{marker}"
+        return url
 
     def _parse_url(self) -> None:
         vcs, url_no_vcs = self.url.split("+", 1)
@@ -429,18 +411,6 @@ class VcsRequirement(FileRequirement):
         self.url = f"{vcs}+{repo}"
         self.repo, self.ref = repo, ref  # type: ignore
 
-    @staticmethod
-    def _build_url_from_req_dict(name: str, url: str, req_dict: RequirementDict) -> str:
-        assert not isinstance(req_dict, str)
-        ref = f"@{req_dict['ref']}" if "ref" in req_dict else ""
-        fragments = f"#egg={urlparse.quote(name)}"
-        if "subdirectory" in req_dict:
-            fragments += (
-                "&subdirectory="  # type: ignore
-                f"{urlparse.quote(req_dict.pop('subdirectory'))}"  # type: ignore
-            )
-        return f"{url}{ref}{fragments}"
-
 
 def filter_requirements_with_extras(
     project_name: str,
@@ -487,7 +457,7 @@ def parse_requirement(line: str, editable: bool = False) -> Requirement:
     else:
         try:
             package_req = PackageRequirement(line)  # type: ignore
-        except (RequirementParseError, InvalidRequirement) as e:
+        except InvalidRequirement as e:
             m = _file_req_re.match(line)
             if m is None:
                 raise RequirementError(str(e)) from None
@@ -505,47 +475,3 @@ def parse_requirement(line: str, editable: bool = False) -> Requirement:
                 " or local directory."
             )
     return r
-
-
-class MockDistribution(Distribution):
-    def __init__(self, data: Setup) -> None:
-        self._data = data
-
-    def read_text(self, filename: str) -> str | None:
-        return None
-
-    def locate_file(self, path: os.PathLike[str] | str) -> os.PathLike[str]:
-        return Path("")
-
-    @property
-    def metadata(self) -> dict[str, Any]:  # type: ignore
-        return {
-            "Name": self._data.name,
-            "Version": self._data.version,
-            "Summary": "UNKNOWN",
-            "Requires-Python": self._data.python_requires,
-        }
-
-    @property
-    def requires(self) -> list[str] | None:
-        result = self._data.install_requires
-        for extra, reqs in self._data.extras_require.items():
-            extra_marker = f"extra == '{extra}'"
-            for req in reqs:
-                parsed = parse_requirement(req)
-                old_marker = str(parsed.marker) if parsed.marker else None
-                if old_marker:
-                    if " or " in old_marker:
-                        new_marker = f"({old_marker}) and {extra_marker}"
-                    else:
-                        new_marker = f"{old_marker} and {extra_marker}"
-                else:
-                    new_marker = extra_marker
-                parsed.marker = Marker(new_marker)
-                result.append(parsed.as_line())
-        return result
-
-
-def parse_metadata_from_source(src_dir: str) -> Distribution:
-    setup = Setup.from_directory(Path(src_dir))
-    return MockDistribution(setup)
